import { Component, ViewChild, DoCheck } from "@angular/core";

import { InternationalisationService } from "../../services/internationalisation/internationalisation.service";
import { LogComponent } from '../../components/log/log.component';
import { FixedVarResults } from "../../results/fixed-var-results";
import { ResultsGraphComponent } from "../results-graph/results-graph.component";
import { CalculatorResults } from "../../results/calculator-results";

@Component({
    selector: "fixedvar-results",
    templateUrl: "./fixedvar-results.component.html",
    styles: [`
    .result_right {
        text-align: right;
    }
    .result_center {
        text-align: center;
    }
    .result_id_0 {
        background-color: #f0f0f0;
    }
    .result_id_1 {
        background-color: #ffffff;
    }
    .result_id_2 {
        font-weight: bolder;
        background-color: #00d0ff;
    }
    `
    ]
})
export class FixedVarResultsComponent implements DoCheck {
    /**
     * résultats non mis en forme
     */
    private _results: FixedVarResults;

    /**
     * true si le graphe des résultats doit être remis à jour
     */
    private _doUpdate: boolean = false;

    private _logUpdated: boolean = false;

    private _graphUpdated: boolean = false;

    /**
     * composant journal
     */
    @ViewChild(LogComponent)
    private logComponent: LogComponent;

    @ViewChild(ResultsGraphComponent)
    private resultsGraphComponent: ResultsGraphComponent;

    constructor(
        private intlService: InternationalisationService
    ) { }

    public set results(rs: CalculatorResults[]) {
        this._results = undefined;
        if (rs != undefined)
            for (const r of rs) {
                if (r instanceof FixedVarResults)
                    this._results = r;
                break;
            }
        this.updateView();
    }

    public updateView() {
        this._logUpdated = false;
        this._graphUpdated = false;
        this.logComponent.log = undefined;

        if (this._results != undefined)
            this._doUpdate = this._results.hasResults;
    }

    public ngDoCheck() {
        if (this._doUpdate)
            this._doUpdate = !this.updateResults();
    }

    /** 
     * met à jour l'affichage des résultats
     * @returns true si les résultats ont pu être mis à jour
     */
    private updateResults() {
        if (this._results != undefined) {
            if (this.resultsGraphComponent != undefined) {
                this.resultsGraphComponent.results = this._results;
                this.resultsGraphComponent.updateView();
                this._graphUpdated = true;
            }
            if (this.logComponent != undefined) {
                this.logComponent.log = this._results.log;
                this._logUpdated = true;
            }
            return this._logUpdated && this._graphUpdated;
        }
        return false;
    }

    /**
     * affichage de la table des résultats fixés
     */
    private get showFixedResults(): boolean {
        return this._results != undefined && this._results.hasFixedResults;
    }

    /**
     * affichage de la table des résultats variés
     */
    private get showVarResults(): boolean {
        return this._results != undefined && this._results.hasVarResults;
    }

    private get uitextParamFixes() {
        return this.intlService.localizeText("INFO_CALCULATOR_PARAMFIXES");
    }

    private get uitextValeurs() {
        return this.intlService.localizeText("INFO_CALCULATOR_VALEURS");
    }

    private getFixedResultClass(i: number) {
        if (this._results.isFixed && i == this._results.fixedResults.length - 1)
            return "font-weight-bold";
        return "result_id_" + String(i & 1);
    }

    private get fixedResults() {
        return this._results.fixedResults;
    }

    private get varResults() {
        return this._results.varResults;
    }

    private get hasResults(): boolean {
        return this.results != undefined && this._results.hasResults;
    }
}
