import { Component, ViewChild, Input, Output, EventEmitter, OnChanges } from "@angular/core";

import { InternationalisationService } from "../../services/internationalisation/internationalisation.service";
import { NgParameter, ParamRadioConfig } from "../../formulaire/ngparam";
import { NgParamInputComponent } from "../ngparam-input/ngparam-input.component";
import { FormulaireService } from "../../services/formulaire/formulaire.service";

@Component({
    selector: "param-field-line",
    templateUrl: "./param-field-line.component.html",
    styles: [
        `.btn-on {
            color:#505050;
            border: 3px solid #505050;
            background-color:white;
            text-transform: uppercase;
            font-size: 0.8em;
        }`,
        `.btn-off {
            color:white;
            border: 3px solid #505050;
            background-color:#505050;
            text-transform: uppercase;
            font-size: 0.8em;
        }`
    ]
})
export class ParamFieldLineComponent implements OnChanges {
    @Input("param")
    private _param: NgParameter;

    @ViewChild(NgParamInputComponent)
    private _ngParamInputComponent: NgParamInputComponent;

    @Output()
    private onValid: EventEmitter<void>;

    /**
     * true si la valeur saisie est valide
     */
    private _isInputValid: boolean = false;

    /**
     * true si le min-max/liste est valide
     */
    private _isRangeValid: boolean = true;

    constructor(private intlService: InternationalisationService,
        private formulaireService: FormulaireService
    ) {
        this.onValid = new EventEmitter();
    }

    private get title(): string {
        let t = "";
        if (this._param.label != undefined)
            t = this._param.label;
        if (this._param.unit != undefined && this._param.unit != "")
            t = t + " (" + this._param.unit + ")";
        return t;
    }

    private get uitextParamFixe() {
        return this.intlService.localizeText("INFO_PARAMFIELD_PARAMFIXE");
    }

    private get uitextParamVarier() {
        return this.intlService.localizeText("INFO_PARAMFIELD_PARAMVARIER");
    }

    private get uitextParamCalculer() {
        return this.intlService.localizeText("INFO_PARAMFIELD_PARAMCALCULER");
    }

    /**
     * Parameter symbol (Q, Ks, B, ...) input attribute
     */
    private get symbol(): string {
        return this._param.symbol;
    }

    /**
     * calcule la présence du radio "paramètre fixé"
     */
    private hasRadioFix(): boolean {
        switch (this._param.radioConfig) {
            case ParamRadioConfig.FIX:
                return false;

            default:
                return true;
        }
    }

    /**
     * calcule la présence du radio "paramètre à varier"
     */
    private hasRadioVar(): boolean {
        switch (this._param.radioConfig) {
            case ParamRadioConfig.VAR:
            case ParamRadioConfig.CAL:
                return true;

            default:
                return false;
        }
    }

    /**
    * calcule la présence du radio "paramètre à calculer"
    */
    private hasRadioCal(): boolean {
        switch (this._param.radioConfig) {
            case ParamRadioConfig.CAL:
                return true;

            default:
                return false;
        }
    }

    /**
    * calcule l'état du radio "paramètre fixé"
    */
    private get radioFixCheck(): string {
        return this.isRadioFixChecked ? "checked" : undefined;
    }

    /**
    * calcule l'état du radio "paramètre à varier"
    */
    private get radioVarCheck(): string {
        return this.isRadioVarChecked ? "checked" : undefined;
    }

    /**
    * calcule l'état du radio "paramètre à calculer"
    */
    private get radioCalCheck(): string {
        if (this._param.radioState == ParamRadioConfig.CAL)
            return "checked";
        return undefined;
    }

    /**
     * retourne l'état du radio "paramètre fixé" sous forme booléenne
     */
    private get isRadioFixChecked(): boolean {
        return this._param.radioState == ParamRadioConfig.FIX;
    }

    /**
     * retourne l'état du radio "paramètre à varier" sous forme booléenne
     */
    private get isRadioVarChecked(): boolean {
        return this._param.radioState == ParamRadioConfig.VAR;
    }

    /*
     * gestion des événements clic sur les radios :
     * envoi d'un message au composant parent
     * cf. https://angular.io/guide/component-interaction#parent-listens-for-child-event
     */

    @Output()
    private onRadio = new EventEmitter<any>();

    private onRadioClick(option: string) {
        this.onRadio.emit({
            "param": this._param,
            "option": option
        });

        // MAJ validité
        this.emitValidity();
    }

    /**
     * désactivation de tous les boutons radio si paramètre par défaut à "CAL"
     */
    private get isDisabled(): boolean {
        return this._param.isDefault && this._param.radioState == ParamRadioConfig.CAL;
    }

    /**
     * désactivation du champ de saisie
     */
    private get isInputDisabled(): boolean {
        return this._param.radioState != ParamRadioConfig.FIX;
    }

    /**
     * classe du radio "fixé"
     */
    private on = true;
    private onClass = "btn-on"
    private offClass = "btn-off"
    private get radioFixClass(): string {
        if (this.on)
            return this.radioFixCheck == undefined ? this.offClass : this.onClass;
        return "";
    }

    /**
     * classe du radio "varier"
     */
    private get radioVarClass(): string {
        if (this.on)
            return this.radioVarCheck == undefined ? this.offClass : this.onClass;
        return "";
    }

    /**
     * classe du radio "calculer"
     */
    private get radioCalClass(): string {
        if (this.on)
            return this.radioCalCheck == undefined ? this.offClass : this.onClass;
        return "";
    }

    /**
     * validité des saisies du composant
     */
    public get isValid(): boolean {
        switch (this._param.radioState) {
            case ParamRadioConfig.FIX:
                return this._isInputValid

            case ParamRadioConfig.VAR:
                return this._isRangeValid;

            default:
                return true;
        }
    }

    /**
     * émission d'un événement de validité
     */
    private emitValidity() {
        this.onValid.emit()
    }

    /**
     * réception d'un événement de NgParamInputComponent
     */
    private onInputChange(event: any) {
        if (event.action == "valid") {
            this._isInputValid = event.value;
            this.emitValidity();
        }
    }

    /**
     * réception d'un événement de validité de ParamValuesComponent
     */
    private onParamValuesValid(event: boolean) {
        this._isRangeValid = event;
        this.emitValidity()
    }

    public ngOnChanges() {
        this._ngParamInputComponent.model = this._param;
        this._ngParamInputComponent.showError = this.isRadioFixChecked;
    }

    private get formHasResults(): boolean {
        return this.formulaireService.currentFormHasResults;
    }
}
